help_docs = \
"""该命令共有子命令：
 ● docs: 显示帮助（后跟 run、status、options 中的一个获取帮助）
 ● run: 运行画图
 ● status: 显示状态
 ● options: 显示配置信息
 ● set: 修改运行配置"""


run_docs = \
"""~draw#run#... 功能：运行画图。

参数格式：
(提词) [反向提词:内置的反向提词] [宽:512] [高:512] [步数:20] [采样器:DPM++ 2M SDE Karras] [cfg 相关值:7] [种子:随机值] [差异种子:随机值] [差异强度:0] [超分启动:0] [超分重绘幅度:0.7] [超分倍数:2] [超分算法:Latent] [超分步数:内部默认值] [临时生效的配置:空]

注意：
1. 括号为必选，方框为存在默认值，冒号后代表默认值。
2. 如果需要使用的方框参数，前面存在使用默认值的方框参数，此时这些参数不能省略，必须使用 / 占位。否则则可以省略。
3. 传递的任何参数，使用 # 隔开

非白名单用户的限制：
1. 正向提词字符串长度 (0, 1000]
2. 反向提词字符串长度 [0, 500]
3. 宽高限制 [100, 960]
4. 总像素不得超过 518400
5. 步数限制 [10, 60]
6. 无权使用高清修复或临时配置功能

⚠ 特别提醒：为防止滥用，已开启 nsfw 过滤机制"""


status_docs = \
"""~draw#status 功能：获取工作状态，包含：启动参数和当前状态。

参数格式：无参数"""


options_docs = \
"""~draw#options 功能：显示配置信息，包含：可用模型、可用 vae、可用 loras、可用采样器、可用超分算法、可用脚本。

参数格式：无参数"""


set_docs = \
"""~draw#set#... 功能：修改配置。（至少需要 su 权限，查看权限可使用：~auth）

参数格式：
(名称1) (值1) (名称2) (值2)...
注意：传递的任何参数，使用 # 隔开

常用的配置项名称：
sd_model_checkpoint, sd_vae, CLIP_stop_at_last_layers"""
